import 'pieces.dart';
import 'color.dart';

const Color BLACK = Color.BLACK;
const Color WHITE = Color.WHITE;

const int EMPTY = -1;

const PieceType PAWN = PieceType.PAWN;
const PieceType KNIGHT = PieceType.KNIGHT;
const PieceType BISHOP = PieceType.BISHOP;
const PieceType ROOK = PieceType.ROOK;
const PieceType QUEEN = PieceType.QUEEN;
const PieceType KING = PieceType.KING;

const Map<String, PieceType> PIECE_TYPES = const {
  'p': PieceType.PAWN,
  'n': PieceType.KNIGHT,
  'b': PieceType.BISHOP,
  'r': PieceType.ROOK,
  'q': PieceType.QUEEN,
  'k': PieceType.KING
};

const String SYMBOLS = 'pnbrqkPNBRQK';

const String DEFAULT_POSITION =
    'rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq - 0 1';

const List POSSIBLE_RESULTS = const ['1-0', '0-1', '1/2-1/2', '*'];

const Map<Color, List<int>> PAWN_OFFSETS = const {
  BLACK: const [16, 32, 17, 15],
  WHITE: const [-16, -32, -17, -15]
};

const Map<PieceType, List> PIECE_OFFSETS = const {
  KNIGHT: const [-18, -33, -31, -14, 18, 33, 31, 14],
  BISHOP: const [-17, -15, 17, 15],
  ROOK: const [-16, 1, 16, -1],
  QUEEN: const [-17, -16, -15, 1, 17, 16, 15, -1],
  KING: const [-17, -16, -15, 1, 17, 16, 15, -1]
};

const List ATTACKS = const [
  20,
  0,
  0,
  0,
  0,
  0,
  0,
  24,
  0,
  0,
  0,
  0,
  0,
  0,
  20,
  0,
  0,
  20,
  0,
  0,
  0,
  0,
  0,
  24,
  0,
  0,
  0,
  0,
  0,
  20,
  0,
  0,
  0,
  0,
  20,
  0,
  0,
  0,
  0,
  24,
  0,
  0,
  0,
  0,
  20,
  0,
  0,
  0,
  0,
  0,
  0,
  20,
  0,
  0,
  0,
  24,
  0,
  0,
  0,
  20,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  20,
  0,
  0,
  24,
  0,
  0,
  20,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  20,
  2,
  24,
  2,
  20,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  2,
  53,
  56,
  53,
  2,
  0,
  0,
  0,
  0,
  0,
  0,
  24,
  24,
  24,
  24,
  24,
  24,
  56,
  0,
  56,
  24,
  24,
  24,
  24,
  24,
  24,
  0,
  0,
  0,
  0,
  0,
  0,
  2,
  53,
  56,
  53,
  2,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  20,
  2,
  24,
  2,
  20,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  20,
  0,
  0,
  24,
  0,
  0,
  20,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  20,
  0,
  0,
  0,
  24,
  0,
  0,
  0,
  20,
  0,
  0,
  0,
  0,
  0,
  0,
  20,
  0,
  0,
  0,
  0,
  24,
  0,
  0,
  0,
  0,
  20,
  0,
  0,
  0,
  0,
  20,
  0,
  0,
  0,
  0,
  0,
  24,
  0,
  0,
  0,
  0,
  0,
  20,
  0,
  0,
  20,
  0,
  0,
  0,
  0,
  0,
  0,
  24,
  0,
  0,
  0,
  0,
  0,
  0,
  20
];

const List RAYS = const [
  17,
  0,
  0,
  0,
  0,
  0,
  0,
  16,
  0,
  0,
  0,
  0,
  0,
  0,
  15,
  0,
  0,
  17,
  0,
  0,
  0,
  0,
  0,
  16,
  0,
  0,
  0,
  0,
  0,
  15,
  0,
  0,
  0,
  0,
  17,
  0,
  0,
  0,
  0,
  16,
  0,
  0,
  0,
  0,
  15,
  0,
  0,
  0,
  0,
  0,
  0,
  17,
  0,
  0,
  0,
  16,
  0,
  0,
  0,
  15,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  17,
  0,
  0,
  16,
  0,
  0,
  15,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  17,
  0,
  16,
  0,
  15,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  17,
  16,
  15,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  1,
  1,
  1,
  1,
  1,
  1,
  1,
  0,
  -1,
  -1,
  -1,
  -1,
  -1,
  -1,
  -1,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  -15,
  -16,
  -17,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  -15,
  0,
  -16,
  0,
  -17,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  -15,
  0,
  0,
  -16,
  0,
  0,
  -17,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  -15,
  0,
  0,
  0,
  -16,
  0,
  0,
  0,
  -17,
  0,
  0,
  0,
  0,
  0,
  0,
  -15,
  0,
  0,
  0,
  0,
  -16,
  0,
  0,
  0,
  0,
  -17,
  0,
  0,
  0,
  0,
  -15,
  0,
  0,
  0,
  0,
  0,
  -16,
  0,
  0,
  0,
  0,
  0,
  -17,
  0,
  0,
  -15,
  0,
  0,
  0,
  0,
  0,
  0,
  -16,
  0,
  0,
  0,
  0,
  0,
  0,
  -17
];

const Map<String, String> FLAGS = const {
  'NORMAL': 'n',
  'CAPTURE': 'c',
  'BIG_PAWN': 'b',
  'EP_CAPTURE': 'e',
  'PROMOTION': 'p',
  'KSIDE_CASTLE': 'k',
  'QSIDE_CASTLE': 'q'
};

const Map<String, int> BITS = const {
  'NORMAL': BITS_NORMAL,
  'CAPTURE': BITS_CAPTURE,
  'BIG_PAWN': BITS_BIG_PAWN,
  'EP_CAPTURE': BITS_EP_CAPTURE,
  'PROMOTION': BITS_PROMOTION,
  'KSIDE_CASTLE': BITS_KSIDE_CASTLE,
  'QSIDE_CASTLE': BITS_QSIDE_CASTLE
};

const int BITS_NORMAL = 1;
const int BITS_CAPTURE = 2;
const int BITS_BIG_PAWN = 4;
const int BITS_EP_CAPTURE = 8;
const int BITS_PROMOTION = 16;
const int BITS_KSIDE_CASTLE = 32;
const int BITS_QSIDE_CASTLE = 64;

const int RANK_1 = 7;
const int RANK_2 = 6;
const int RANK_3 = 5;
const int RANK_4 = 4;
const int RANK_5 = 3;
const int RANK_6 = 2;
const int RANK_7 = 1;
const int RANK_8 = 0;

const Map SQUARES = const {
  'a8': 0,
  'b8': 1,
  'c8': 2,
  'd8': 3,
  'e8': 4,
  'f8': 5,
  'g8': 6,
  'h8': 7,
  'a7': 16,
  'b7': 17,
  'c7': 18,
  'd7': 19,
  'e7': 20,
  'f7': 21,
  'g7': 22,
  'h7': 23,
  'a6': 32,
  'b6': 33,
  'c6': 34,
  'd6': 35,
  'e6': 36,
  'f6': 37,
  'g6': 38,
  'h6': 39,
  'a5': 48,
  'b5': 49,
  'c5': 50,
  'd5': 51,
  'e5': 52,
  'f5': 53,
  'g5': 54,
  'h5': 55,
  'a4': 64,
  'b4': 65,
  'c4': 66,
  'd4': 67,
  'e4': 68,
  'f4': 69,
  'g4': 70,
  'h4': 71,
  'a3': 80,
  'b3': 81,
  'c3': 82,
  'd3': 83,
  'e3': 84,
  'f3': 85,
  'g3': 86,
  'h3': 87,
  'a2': 96,
  'b2': 97,
  'c2': 98,
  'd2': 99,
  'e2': 100,
  'f2': 101,
  'g2': 102,
  'h2': 103,
  'a1': 112,
  'b1': 113,
  'c1': 114,
  'd1': 115,
  'e1': 116,
  'f1': 117,
  'g1': 118,
  'h1': 119
};

const int SQUARES_A1 = 112;
const int SQUARES_A8 = 0;
const int SQUARES_H1 = 119;
const int SQUARES_H8 = 7;
